/*
 * dummy_plugin.h - empty plugin which is used as fallback if a plugin couldn't
 *                  be found
 *
 * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
 * 
 * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program (see COPYING); if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 */


#ifndef _DUMMY_PLUGIN_H
#define _DUMMY_PLUGIN_H

#include "plugin.h"
#include "plugin_view.h"


class dummyPlugin : public plugin
{
public:
	inline dummyPlugin( void ) :
		plugin( NULL, NULL )
	{
	}

	inline virtual ~dummyPlugin()
	{
	}


	inline virtual void saveSettings( QDomDocument &, QDomElement & )
	{
	}

	inline virtual void loadSettings( const QDomElement & )
	{
	}

	inline virtual QString nodeName( void ) const
	{
		return( "dummyplugin" );
	}

protected:
	virtual pluginView * instantiateView( QWidget * _parent )
	{
		return( new pluginView( this, _parent ) );
	}

} ;


#endif
